#######################################################################
#
# edglcg for managing top-level EDG/LCG configuration files
#
#                 Currently managed files:
#                 /etc/sysconfig/edg
#                 /etc/sysconfig/lcg
#                 LCG environment scripts (lcgenv)
#
#
# Novak Judit <judit.novak@cern.ch>, German Cancio <German.Cancio@cern.ch>, Thorsten Kleinwort <Thorsten.Kleinwort@cern.ch>
# Version 0.2.8 : 17/12/04 07:08
#
# ** Generated file : do not edit **
#
#######################################################################

package NCM::Component::edglcg;

use strict;
use NCM::Component;
use vars qw(@ISA $EC);
@ISA = qw(NCM::Component);
$EC=LC::Exception::Context->new->will_store_all;
use NCM::Check;
use File::Copy;

use EDG::WP4::CCM::Element;


local(*DTA);

my $comment="#\n# generated automatically by quattor NCM edglcg component\n# DO NOT EDIT\n#\n";

sub _update_file {
  my ($self,$fname,$contents)=@_;

  if ( ! -e $fname ) {
    open ( CONF,">$fname" );
    print CONF $contents;
    close (CONF);
    $self->log("$fname created");
  } else {
    my $result = LC::Check::file( $fname,
				  backup => ".old",
				  contents => $contents
				);
    $self->log("$fname updated") if $result;
  }
  return 1;
}




sub _conffile {
  my ($self, $config,$type) = @_;

  my $type_uc=uc($type);
  # global conf. vars 
  my %global;
  my $edge = $config->getElement("/system/$type/config");
  while ( $edge->hasNextElement() ) {
    my $element = $edge->getNextElement();
    if ( $element->isProperty() ) {
      my $name = $element->getName();
      my $value = $element->getValue();
      $global{$name} = $value;
    }
  }
	
  # Checking for obligatory variables
  unless ( $global{$type_uc.'_LOCATION'} ) {
    $self->error( "Couldn't determine variable ".
		  $type_uc."_LOCATION" ) ; 
    return
  }	
  unless ( $global{$type_uc.'_TMP'} ) {
    $self->error( "Couldn't determine variable ".
		  $type_uc."_TMP" ) ;
  }	

  if ( ! $global{$type_uc.'_LOCATION_VAR'} ) {	
    $global{$type_uc.'_LOCATION_VAR'} = 
      $global{$type_uc.'_LOCATION'}; 
  }

  # Generate directory if not existing
  # >>> should use a more sophisticated function and
  #     error recovery
  unless (-d $global{$type_uc.'_LOCATION_VAR'}) {
    `/bin/mkdir -p $global{$type_uc.'_LOCATION_VAR'}`;
  }

  # Generate contents

  my $contents = "";
  my $actual = "";
  my $datafile = "/usr/lib/ncm/config/edglcg/sysconfig_template_".$type;

  my $in;
  open ( DTA, "< $datafile" );
  while ($in=<DTA>) {
    if ( $in =~ /^#\s*(\w+)/ && exists($global{"$1"}) ) {
      my $var = $1;
      $contents .= $in;
      while ($in=<DTA>) {
	last if ($in =~ /^\s*\n$/);
	$contents .= $in;
      }
      $contents .= $var. "=". $global{"$var"}."\n\n";
    } else {
      $contents .= $in;
    }
  }
  close (DTA);

  # This location should not be changed!
  # All EDG/LCG configuration relies
  # on finding this file in exactly this place.
  my $sysconf="/etc/sysconfig/".$type;

  $contents=$comment.$contents;
  $self->_update_file ($sysconf,$contents);
}


sub _update_env {
  my ($self,$config)=@_;

  # env conf. vars
  my %env;
  if ($config->elementExists("/software/components/edglcg/env")) {
    my $edge = $config->getElement("/software/components/edglcg/env");
    while ( $edge->hasNextElement() ) {
      my $element = $edge->getNextElement();
      if ( $element->isProperty() ) {
	my $name = $element->getName();
	my $value = $element->getValue();
	$env{$name} = $value;
      }
    }
  }
  my ($var,$sh,$csh);
  $sh="#!/bin/sh\n";
  $csh="#!/bin/csh\n";
  $sh .= $comment;
  $csh.= $comment;
  foreach $var (keys %env) {
    $sh .= 'export '.$var.'='.$env{$var}."\n";
    $csh.='setenv '.$var .' '.$env{$var}."\n";
  }

  $self->_update_file ('/etc/profile.d/lcgenv.sh',$sh);
  $self->_update_file ('/etc/profile.d/lcgenv.csh',$csh);

  chmod(0755,'/etc/profile.d/lcgenv.sh');
  chmod(0755,'/etc/profile.d/lcgenv.csh');

#
# Setting up local s/w directories, if any
#
  my $vo;
  my $is_local;
  my $local_dir;
  my $gid;
  my $VO_NAME;

  if ($config->elementExists("/software/components/edglcg/swpath")) {
      my $swpath = $config->getElement("/software/components/edglcg/swpath");
      while ( $swpath->hasNextElement() ) {
	  $vo = $swpath->getNextElement();
	  $VO_NAME = uc($vo->getName());
	  $is_local = $config->getValue($vo->getPath()->toString()."/local");
	  
	  if ($config->getValue($vo->getPath()->toString()."/local") eq "true"){
	      $local_dir = $env{"VO_".$VO_NAME."_SW_DIR"};
	      $local_dir = $config->getValue($vo->getPath()->toString()."/path") if ($local_dir eq "");
	      $gid       = $config->getValue($vo->getPath()->toString()."/gid");
	      
	      $self->error("no local s/w path defined for ".$VO_NAME."!") if ($local_dir eq "");
	      
	      LC::Check::directory($local_dir);
	      
	      LC::Check::status($local_dir,
				owner => "root",
				group => $gid,
				mode  => 0775 );
	      
	  }
      }
  }
}

##########################################################################
sub Configure($$@) {
##########################################################################

  my ($self, $config) = @_;
  #
  # handle sysconfig files
  #
  foreach ('edg','lcg') {
    $self->_conffile($config,$_);
  }

  #
  # handle env files, including info service location
  #
  $self->_update_env($config);

}
1; 	# Required for PERL modules
